/*
-- Company: 		Trenz Electronic
-- Engineer: 		Oleksandr Kiyenko / John Hartfiel
 */
#include "te_si539x.h"

#ifdef CLOCK_SI5395

#include "te_uart.h"

#include "te_Si5395-Registers.h"
#ifdef PLL_B
  #include "te_Si5395-Registers_B.h"
#endif




int si539x_version(unsigned char chip_addr){
	unsigned char val;
	int Status;
	  Status = iic_read16(chip_addr, 0x0003, &val);
	  xil_printf("SI%x",val);

	  Status = iic_read16(chip_addr, 0x0002, &val);
	  xil_printf("%x",val);

	  Status = iic_read16(chip_addr, 0x0005, &val);
    if (val==0) {
      xil_printf("-A\r\n");
    } else if (val==1) {
      xil_printf("-B\r\n");
    } else if (val==2) {
      xil_printf("-C\r\n");
    } else if (val==3) {
      xil_printf("-D\r\n");
    } else {
      xil_printf("-%x\r\n",val);
    }

	  return Status;
}

int si539x_status_wait(unsigned char chip_addr){
	unsigned char valF,valC,valE,valD,val11;
	unsigned int  cnt=0, tmp;
  unsigned char val;
  unsigned short addr;
	int Status;
  int i;
  
  (void)usleep(0x5U);
  
  Status = iic_read16(chip_addr, 0x000C, &valC);
  Status = iic_read16(chip_addr, 0x000E, &valE);
  Status = iic_read16(chip_addr, 0x000D, &valD);
  Status = iic_read16(chip_addr, 0x0011, &val11);
  tmp = 1;
  // Wait until internal calibration is not busy
  while (tmp ==1) {
    cnt=cnt+0x100U ;
    (void)usleep(0x100U);
    Status = iic_read16(chip_addr, 0x000C, &valC);
    Status = iic_read16(chip_addr, 0x000F, &valF);
    tmp =((((valF) & (0x20))>>5) | (valC) & (0x01));
    if ((cnt % 0x100U) == 0) {
      xil_printf("Status   0xC:0x%x, 0xE:0x%x, 0xD:0x%x, 0x11:%x, 0xF:%x (...waiting for calibration...%i us).\r",valC,valE,valD,val11,valF,cnt);
    }
    if (cnt >= TIME_CHECK_PLL_CONFIG_US) {
        xil_printf("Status   0xC:0x%x, 0xE:0x%x, 0xD:0x%x, 0x11:%x, 0xF:%x (...calibration not finished after %i us...exit...).\r\n",valC,valE,valD,val11,valF,cnt);
        xil_printf("Status 0xC:0x%x(cal bit0:%i) and 0xF:%x(cal bit5:%i) will be checked one time again after %i us\r\n",valC,((valC) & (0x01)),valF,(((valF) & (0x10))>>5),DELAY_AFTER_PLL_CONFIG_US);
        tmp = 0;
    }
  }
  
   //sleep need for PCIe
  (void)usleep(DELAY_AFTER_PLL_CONFIG_US);
    
  Status = iic_read16(chip_addr, 0x000F, &valF);
  Status = iic_read16(chip_addr, 0x000C, &valC);
  Status = iic_read16(chip_addr, 0x000E, &valE);
  Status = iic_read16(chip_addr, 0x000D, &valD);
  Status = iic_read16(chip_addr, 0x0011, &val11);
  xil_printf("PLL Status Register   0xC:0x%x, 0xE:0x%x, 0xD:0x%x, 0x11:%x, 0xF:%x.                                                     \r\n",valC,valE,valD,val11,valF);


// commented for test purpose only:
  // xil_printf("Test CPLD Firmware only-----\r\n");
  for (i=0; i<12; ++i) {
    val=outputEnableReg[i].value;
    addr=outputEnableReg[i].address;
    Status = iic_write16(chip_addr, addr, val);
    
    (void)usleep(DELAY_ENABLE_OUTPUTS);			

    // xil_printf("Test OE Register chip 0x%x: Add 0x%x Val 0x%x  .\r\n",chip_addr, addr, val);
		
  }
// commented for test purpose end
	return Status;
}

int si539x_reset(unsigned char chip_addr){
	int i, Status;
	unsigned char val;
	unsigned short addr;
  
    //test hard power down+reset todo hier weiter und variable machen --> kommt auhc fehlermeldung aber einige register sind dann anders und auch die ausgangs clk.
    val = 0x03;
    addr = 0x001E;
    
    Status = iic_write16(chip_addr, addr, val);
    if(Status != XST_SUCCESS) {
        p_printf(("Return code %i.\n", Status));
    }
    
    p_printf(("TODO Configurable:device  0x%02X: Reset and power down PLL write 0x%02X to address 0x%04X.\n",chip_addr, val, addr));


	return XST_SUCCESS;
}


int si539x_recal(unsigned char chip_addr){
	int i, Status;
	unsigned char val;
	unsigned short addr;
  
    Status = iic_write16(chip_addr, 0x0B24, 0xC0);
    Status = iic_write16(chip_addr, 0x0B25, 0x00);
    Status = iic_write16(chip_addr, 0x0540, 0x01);
    p_printf(("Sleep for cal after, sleep in %d us .\r\n", DELAY_AFTER_PLL_PREAMPLE));
    (void)usleep(DELAY_AFTER_PLL_PREAMPLE);
    p_printf(("Sleep finished sleep.\r\n"));
    
    Status = iic_write16(chip_addr, 0x0514, 0x01);
    Status = iic_write16(chip_addr, 0x001C, 0x01);
    Status = iic_write16(chip_addr, 0x0540, 0x00);
    Status = iic_write16(chip_addr, 0x0B24, 0xC3);
    Status = iic_write16(chip_addr, 0x0B25, 0x02);

    
    p_printf(("Recal finished :device  0x%02x \n",chip_addr));


	return XST_SUCCESS;
}



int si539x_init(unsigned char chip_addr, unsigned char readonly, unsigned char file){
	int i, Status;
	unsigned char val;
	unsigned short addr;

	si5395_reva_register_t const *config_register;
	unsigned int config_register_length=0;


    if (readonly==0) {
      p_printf(("Si539x Init Registers Write.\r\n"));
    } else {
      p_printf(("Si539x Init Registers Read only.\r\n"));
      
    }
    if (file=='A') {
	#ifdef SI5395_REVA_REG_CONFIG_NUM_REGS
    	config_register= &si5395_reva_registers;
    	config_register_length= SI5395_REVA_REG_CONFIG_NUM_REGS;
	#endif
    }
    if (file=='B') {
	#ifdef SI5395_REVA_REG_CONFIG_NUM_REGS_B
    	config_register= &si5395_reva_registers_b;
    	config_register_length= SI5395_REVA_REG_CONFIG_NUM_REGS_B;
	#endif
    }


  #ifdef DEBUG_REG
    unsigned char readback_test;
  #endif
    for (i=0; i<config_register_length; ++i) {
        val = config_register[i].value;
        addr = config_register[i].address;
        if (readonly==0) {
          
// commented for test purpose only:
  // xil_printf("Test CPLD Firmware only-----\r\n");
          //save output registers and disable output (workaround)
          if (addr==0x0103) {
            outputEnableReg[0].address=config_register[i].address;
            outputEnableReg[0].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          }
          if (addr==0x0108) {
            outputEnableReg[1].address=config_register[i].address;
            outputEnableReg[1].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          }
          if (addr==0x010D) {
            outputEnableReg[2].address=config_register[i].address;
            outputEnableReg[2].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          }
          if (addr==0x0112) {
            outputEnableReg[3].address=config_register[i].address;
            outputEnableReg[3].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          } 
          if (addr==0x0117) {
            outputEnableReg[4].address=config_register[i].address;
            outputEnableReg[4].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          }
          if (addr==0x011C) {
            outputEnableReg[5].address=config_register[i].address;
            outputEnableReg[5].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          }
          if (addr==0x0121) {
            outputEnableReg[6].address=config_register[i].address;
            outputEnableReg[6].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          }
          if (addr==0x0126) {
            outputEnableReg[7].address=config_register[i].address;
            outputEnableReg[7].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          }
          if (addr==0x012B) {
            outputEnableReg[8].address=config_register[i].address;
            outputEnableReg[8].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          }
          if (addr==0x0130) {
            outputEnableReg[9].address=config_register[i].address;
            outputEnableReg[9].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          }
          if (addr==0x0135) {
            outputEnableReg[10].address=config_register[i].address;
            outputEnableReg[10].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          }
          if (addr==0x013A) {
            outputEnableReg[11].address=config_register[i].address;
            outputEnableReg[11].value=config_register[i].value;
            //disable output --> in case it is enable, it will restored after calibration
            val = 0x00;
          }
// commented for test purpose end
          
          //write
          Status = iic_write16(chip_addr, addr, val);
          if(Status != XST_SUCCESS) {
              p_printf(("Couldn't write value 0x%02X to address 0x%04X.\n", val, addr));
              p_printf(("There were %d successful I2C writes so far.\n", i));
              return Status;
          }
          
          if ( addr==0x0540 && i<10) {
            //break only on the first time
            // p_printf(("Sleep for PREAMPLE (loop %d, last reg which was wrote 0x%x, sleep in %d us .\r\n", i,addr, DELAY_AFTER_PLL_PREAMPLE));
            (void)usleep(DELAY_AFTER_PLL_PREAMPLE);
            // p_printf(("Sleep finished sleep.\r\n"));
          }

          
          (void)usleep(DELAY_PER_WRITE_ACCESS);
          if ( i+1<config_register_length ) {
            if (config_register[i+1].address==0x0514 ) {
              //break only on the first time
              // p_printf(("Sleep for POSTAMPLE (loop %d, last reg which was wrote 0x%x, sleep in %d us .\r\n", i,addr, DELAY_BEFOR_PLL_POSTAMPLE));
              (void)usleep(DELAY_BEFOR_PLL_POSTAMPLE);
              // p_printf(("Sleep finished sleep.\r\n"));
            }
          }
        }
        // //Test 
        // unsigned char valF,valC,valE,valD,val11;
        // Status = iic_read16(chip_addr, 0x000F, &valF);
        // Status = iic_read16(chip_addr, 0x000C, &valC);
        // Status = iic_read16(chip_addr, 0x000E, &valE);
        // Status = iic_read16(chip_addr, 0x000D, &valD);
        // Status = iic_read16(chip_addr, 0x0011, &val11);
        // xil_printf("Test 3 PLL Status Register   0xC:0x%x, 0xE:0x%x, 0xD:0x%x, 0x11:%x, 0xF:%x.                                                     \r\n",valC,valE,valD,val11,valF);
        // //Test finished
            
        #ifdef DEBUG_REG
        if (addr!=0x0B24 && addr!=0x0B25  && addr!=0x0540  && addr!=0x0514  && addr!=0x001C) {
          Status = iic_read16(chip_addr, addr, &readback_test);
          if(Status != XST_SUCCESS) {
              p_printf(("Couldn't write value 0x%02X to address 0x%04X.\n", val, addr));
              p_printf(("There were %d successful I2C writes so far.\n", i));
              return Status;
          }

          if (val != readback_test) {
            p_printf(("device  0x%02X:address 0x%04X: Write 0x%02X to  and read 0x%02X  ----- Difference detected please check.\r\n",chip_addr,addr, val, readback_test));
          } else {
            p_printf(("device  0x%02X:address 0x%04X: Write 0x%02X to  and read 0x%02X.\r\n",chip_addr,addr, val, readback_test));
          }
        }
        #endif
        
    }
    

    
    if (readonly==0) {
      p_printf(("Si539x device  0x%02X: Init Complete.\r\n",chip_addr));
    } else {
      p_printf(("Si539x device  0x%02X: Read only Complete.\r\n",chip_addr));
      
    }
	
    
#ifdef NVM_CODE
    p_printf(("Write this configuration to the Si5395 NVM? (y/N)\r\n"));
    if(uart_read_char() == 'y'){
        si539x_write_nvm(chip_addr)
    }
#endif
	return XST_SUCCESS;
}


#ifdef NVM_CODE
int si539x_write_nvm(unsigned char chip_addr){
	#ifndef NVM_REALY
	#error "To relase NVM write function please define NVM_REALY in te_si539x.h. You do it on your own risk!"
	#endif

#ifdef NVM_REALY
	int Status;
	unsigned char val;
	unsigned char active_nvm_bank;

    p_printf(("Si539x NVM Programming Start.\r\n"));

    Status = iic_read16(chip_addr, ACTIVE_NVM_BANK, &val);
    if(Status != XST_SUCCESS){
    	p_printf(("Error: Si539x Register read failed.\r\n"));
    	return Status;
    }
    active_nvm_bank = val;
    if(!((active_nvm_bank == 3) | (active_nvm_bank == 15))){
    	p_printf(("Error: Wrong ACTIVE_NVM_BANK d (should be 3 or 15).\r\n", active_nvm_bank));
    	return XST_FAILURE;
    }

    p_printf(("Write NVM_WRITE register.\r\n"));
    Status = iic_write16(chip_addr, NVM_WRITE, 0xC7);
    if(Status != XST_SUCCESS){
    	p_printf(("Error: Si539x Register write failed\r\n"));
    	return Status;
    }

    p_printf(("Poll DEVICE_READY.\r\n"));
    do{
    	Status = iic_read16(chip_addr, DEVICE_READY, &val);
    	if(Status != XST_SUCCESS){
    		p_printf(("Error: Si539x Register read failed\r\n"));
    		return Status;
    	}
    }
	while(val != 0x0F);

    p_printf(("Load the NVM contents into non-volatile memory.\r\n"));
    Status = iic_write16(chip_addr, NVM_READ_BANK, 0x01);
    if(Status != XST_SUCCESS){
    	p_printf(("Error: Si539x Register write failed\r\n"));
    	return Status;
    }

    p_printf(("Poll DEVICE_READY.\r\n"));
    do {
    	Status = iic_read16(chip_addr, DEVICE_READY, &val);
    	if(Status != XST_SUCCESS){
    		p_printf(("Error: Si539x Register read failed.\r\n"));
    		return Status;
    	}
    }
    while(val != 0x0F);

    p_printf(("Read ACTIVE_NVM_BANK.\r\n"));
    Status = iic_read16(chip_addr, ACTIVE_NVM_BANK, &val);
    if(Status != XST_SUCCESS){
    	p_printf(("Error: Si539x Register read failed.\r\n"));
    	return Status;
    }
    if(((active_nvm_bank << 2) | 0x03) != val){
    	p_printf(("Error: Wrong ACTIVE_NVM_BANK %d (was %d next should be %d).\r\n", val, active_nvm_bank, ((active_nvm_bank << 2) | 0x03)));
    	return XST_FAILURE;
    }

    p_printf(("Si539x NVM Programming Complete.\r\n"));
	#endif
	return XST_SUCCESS;
}

#endif
#endif /*Clock chip*/
